#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function

from .boolean import boolean
from .builtins import items
from .encoding import UnicodeTransformChar

__all__ = (
    'boolean',
    'items',
    'UnicodeTransformChar',
)


def pretty_print_req(req):
    """
    At this point it is completely built and ready
    to be fired; it is "prepared".

    However pay attention at the formatting used in
    this function because it is programmed to be pretty
    printed and may differ from the actual request.
    """
    return ('{}\n{}\n{}\n\n{}'.format(
        '-----------START-----------',
        req.method + ' ' + req.url,
        '\n'.join(
            '{}: {}'.format(k, v) for k, v in req.headers.items()
        ),
        req.body,
    ))

"""

def timed_function(func):
    @wraps(func)
    def timed_function_wrapper(*args, **kwargs):
        time_start = timeit.default_timer()
        result = func(*args, **kwargs)
        elapsed = round(Decimal(timeit.default_timer() - time_start), 4)
        if isinstance(result, AvailabilityResults):
            result.elapsed = elapsed
            return result
        return AvailabilityResults(result, elapsed)

    return timed_function_wrapper
"""



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function

import simplejson as json

from collections import OrderedDict


def _complex_encode(obj):
    return '{0}'.format(obj)


def jsonify(data, pretty=False, **kwargs):
    """Serialize Python objects to JSON with optional 'pretty' formatting

    Raises:
        TypeError: from :mod:`json` lib
        ValueError: from :mod:`json` lib
        JSONDecodeError: from :mod:`json` lib
    """
    isod = isinstance(data, OrderedDict)
    params = {
        'for_json': True,
        'default': _complex_encode,
    }
    if pretty:
        params['indent'] = 2
        params['sort_keys'] = False if isod else True
    params.update(kwargs)
    try:
        return json.dumps(data, ensure_ascii=False, **params)
    except UnicodeDecodeError:
        return json.dumps(data, **params)


def jsonexpand(data, ordered=True):
    if ordered:
        return json.loads(data, object_pairs_hook=OrderedDict)
    return json.loads(data)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals, division

import os

from collections import OrderedDict
from armory.utils import items


_BOOL_MAP = {
    'y': True,
    'yes': True,
    't': True,
    'true': True,
    '1': True,
    'n': False,
    'no': False,
    'f': False,
    'false': False,
    '0': False,
}
_NOT_PROVIDED = object()
_ENV_ERROR_MSG = 'Environment variable "{0}" or a default value is required'


def boolean(value, boolmap=_BOOL_MAP):
    """ Convert string value to <type bool>. """
    return boolmap.get(value.lower(), bool(value))


def env(key, default=_NOT_PROVIDED, cast=str,
        force=False, boolmap=False, sticky=False):
    """
    Retrieve environment variables and specify default and options.

    :param key: (required) environment variable name to retrieve
    :param default: value to use if the environment var doesn't exist
    :param cast: values always come in as strings, cast to this type if needed
    :param force: force casting of value even when it may not be needed
    :param boolmap: if True use default map, otherwise you can pass custom map
    :param sticky: injects default into environment so child processes inherit

    NOTE: None can be passed as the default to avoid raising a ValueError
    """
    if boolmap is True:
        boolmap = _BOOL_MAP

    value = os.environ.get(key, default)
    if value is _NOT_PROVIDED:
        raise ValueError(_ENV_ERROR_MSG.format(key))

    if sticky and value == default:
        try:
            os.environ[key] = value
        except TypeError:
            os.environ[key] = str(value)

    if force or (value != default and type(value) != cast):
        if cast is bool and boolmap:
            value = boolean(value, boolmap=boolmap)
        else:
            value = cast(value)

    return value


'''


def env(key, default=NOT_PROVIDED, cast=str, force=False):
    """utility for getting settings from environment variables"""
    if default is NOT_PROVIDED:
        val = os.environ.get(key)
        if val is None:
            errmsg = "Environment variable {0} is required."
            raise ValueError(errmsg.format(key))
    else:
        val = os.environ.get(key, default)
    if val is not None:
        if val == default and not force:
            return val
        else:
            return cast(val)
    return val
'''
NOT_PROVIDED = object()
NONEOBJ = object()


class VarInfo(object):
    def __init__(self, default, cast, lazy, force):
        self.default = default
        self.cast = cast
        self.lazy = lazy
        self.force = force

    def json_dict(self):
        jd = OrderedDict()
        jd['default'] = '{0}'.format(self.default)
        jd['cast'] = '{0}'.format(self.cast)
        jd['lazy'] = self.lazy
        jd['force'] = self.force
        return jd


class Var(object):
    def __init__(self, name, default=NOT_PROVIDED, cast=None, lazy=None):
        self.name = name
        self.default = default
        self.cast = cast
        self.lazy = lazy

    def _var_info(self):
        return VarInfo(self.default, self.cast, self.lazy)


'''
TODO:
*   make sure there is the option to only coerce values retrieved, but leave
    a provided default at whichever type it currently is
*   provide subclass options to specify env vars and validate that the env var
    actually conforms to a specification (glob/regex/url/path/choice)
'''


class Environment(object):
    """
    Alternative class-based environment config implementation

    Allows you to define information about the environment under which a
    program should be running using a dictionary. The benefit of doing
    this instead of using the function is that you can define all of the things
    that you expect from the environment up front so that it is easier for
    someone looking at the code to understand what variables your code needs.
    """

    def __init__(self, varconf, cast=str, force=False, lazy=True):
        self._varconf = varconf
        self._cast = cast
        self._force = force
        self._lazy = lazy
        self._variables = {}
        self._values = {}
        if isinstance(varconf, list):
            try:
                for conf in varconf:
                    self._variables[conf.name] = conf._var_info()
            except AttributeError:
                raise ValueError('ERROR: expected list of Var objects')
        else:
            for var, conf in items(varconf):
                self._variables[var] = VarInfo(
                    default=conf.get('default', NOT_PROVIDED),
                    cast=conf.get('cast', self._cast),
                    lazy=conf.get('lazy', self._lazy),
                    force=conf.get('force', self._force)
                )
        if lazy is False:
            self._resolve_all()

    def __call__(self, varname):
        return self._resolve(varname)

    def _resolve(self, varname):
        try:
            return self._values[varname]
        except KeyError:
            pass

        varinfo = self._variables.get(varname, NONEOBJ)
        if varinfo is NONEOBJ:
            raise ValueError('ERROR: unknown var "{0}"'.format(varname))

        if varinfo.default is NOT_PROVIDED:
            value = os.environ.get(varname)
            if value is None:
                errmsg = "Environment variable '{0}' is required!"
                raise ValueError(errmsg.format(varname))
        else:
            value = os.environ.get(varname, varinfo.default)

        if value is not None:
            if value != varinfo.default or varinfo.force:
                value = varinfo.cast(value)

        self._values[varname] = value
        return value

    def _resolve_all(self):
        for var in self._variables.keys():
            self._resolve(var)



# -*- encoding: utf-8 -*-
from __future__ import absolute_import

import requests
import json
import logging

from bs4 import BeautifulSoup as htmldoc


def carrier_lookup():
    return None


class CarrierLookup(object):
    def __init__(self, number, logname=None):
        self.number = number
        self._logname = logname if logname else ''
        self.log = logging.getLogger(self._logname)

    def lookup(self):
        log = self.log

        domain = 'www.twilio.com'
        host = 'https://{0}'.format(domain)
        lookup = '{0}/lookup'.format(host)

        # masquerade as OS-X Firefox
        s = requests.Session()
        s.headers['user-agent'] = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:37.0) Gecko/20100101 Firefox/37.0'
        s.headers['x-requested-with'] = 'XMLHttpRequest'
        s.headers['accept-language'] = 'en-US,en;q=0.5'
        s.headers['cache-control'] = 'no-cache'
        s.headers['content-type'] = 'application/x-www-form-urlencoded; charset=UTF-8'
        s.headers['host'] = domain
        s.headers['DNT'] = '1'
        s.headers['connection'] = 'close'

        # fetch the base page to set the cookies and get csrf and sid values
        r = s.get(lookup)
        hdrs = {k: v for k, v in s.headers.iteritems()}
        cookies = [{c.name: c.value} for c in s.cookies]
        log.debug('\nsession headers: {0}\n'.format(jsonify(hdrs)))
        log.debug('\nsession cookies: {0}\n'.format(jsonify(cookies)))
        if not cookies:
            log.error('unknown error accessing base page: {0}'.format(lookup))
            log.error('ERROR: {0}'.format(r.status_code))
            log.error(r.text)
            raise ValueError()

        # extract the csrf and sid
        page = htmldoc(r.text)
        token = page.find('meta', attrs={'name': 'csrfToken'})
        if token is None:
            log.debug(r.text)
        csrf = token['content']
        log.debug('NAME={0} CONTENT={1}'.format(token['name'], csrf))
        sid_attrs = {'type': 'hidden', 'role': 'visitorSid'}
        role = page.find('input', attrs=sid_attrs)
        sid = role['value']
        log.debug('ROLE={0} VALUE={1}'.format(role['role'], sid))

        # retrieve the phone number information
        s.headers['referer'] = lookup
        params = {
            'Type': 'lookup',
            'PhoneNumber': "{0}".format(self.number),
            'VisitorSid': sid,
            'CSRF': csrf,
        }
        log.debug('\nparams: {0}\n'.format(jsonify(params)))
        url = '{0}/functional-demos'.format(host)
        r = s.post(url, params=params)
        info = json.loads(r.content)
        return info



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function


def items(dict_obj):
    try:
        return dict_obj.iteritems()
    except AttributeError:
        return dict_obj.items()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

try:
    input = raw_input
except:
    pass

import click
import os
import logging
import json

from collections import OrderedDict

from armory.serialize import jsonify
from armory.phone.phone import PhoneNumber, EmailSMS

CONFIG_FILE = '.phonecache'

helptxt = {}
helptxt['config_file'] = 'specify the config file location'
helptxt['list_lookups'] = 'list the currently stored lookups'

#@click.argument('import_path', click.Path(exists=True), required=False)
@click.group(invoke_without_command=True)
@click.option(
    '-c', '--config', 'config',
    help=helptxt.get('config_file'))
@click.option(
    '-l', '--list', 'list_lookups',
    is_flag=True, default=False, #is_eager=True,
    help=helptxt.get('list_lookups'))
@click.version_option(message='%(prog)s %(version)s')
@click.pass_context
def CLI(ctx, config, list_lookups):
    """Lookup basic phone number information or send SMS messages"""
    loglevel = 'info'
    verbosity = getattr(logging, loglevel.upper(), 'INFO')
    #verbosity = logging.DEBUG
    ctx.obj = {
        'verbosity': verbosity,
        'logfile': None,
        'config': {'lookups': {}},
    }
    #logfmt = '%(levelname)-8s | %(message)s'
    logfmt = '%(message)s'
    logging.basicConfig(format=logfmt, level=verbosity)
    debug('CLI >  CWD="{0}"'.format(os.getcwd()))
    with open(CONFIG_FILE, 'ab+') as phonedb:
        phonedb.seek(0)
        rawdata = phonedb.read()
        if rawdata:
            data = json.loads(rawdata, object_pairs_hook=OrderedDict)
            debug(data)
            ctx.obj['config'] = data
            if 'lookups' not in ctx.obj['config']:
                ctx.obj['config']['lookups'] = {}

    if 'sms_gateways' not in ctx.obj['config']:
        """
        gwsorted = sorted(ctx.obj['config']['sms_gateways'].items())
        gwordered = OrderedDict()
        for (k, v) in gwsorted:
            gwordered[k] = v
        ctx.obj['config']['sms_gateways'] = gwordered
        """
        info('loading SMS gateways...')
        sms_gateways = {}
        with open('carrier-gateways', 'rb') as gateways:
            gwdata = gateways.read()
            import re
            re_carrier = r'(?P<carrier>\w+[\w\&\(\)\-\'\s]*[\w\)\(]+)\s+'
            re_gateway = r'10digitphonenumber(?P<gateway>\@\w[\w\.]*\w)\s+'
            gwre = re.compile(re_carrier + re_gateway)
            gws = gwre.findall(gwdata)
            for gw in gwre.finditer(gwdata):
                gwinfo = gw.groupdict()
                debug(gw.groupdict())
                sms_gateways[gwinfo['carrier']] = gwinfo['gateway']
            num_lines = len([l for l in gwdata.split('\n') if len(l)])
            debug('lines={0} ({1} / 2)'.format(num_lines/2, num_lines))
            debug('matches={0}'.format(len(data)))
        ctx.obj['config']['sms_gateways'] = sms_gateways
        with open(CONFIG_FILE, 'wb') as cfg:
            cfg.write(json.dumps(ctx.obj['config'], indent=2))
        info('SMS gateways written to config')

    if list_lookups:
        sms_updated = False
        from fuzzywuzzy import fuzz
        from fuzzywuzzy import process
        lookups = ctx.obj['config']['lookups']
        info('cached lookups:')
        for number, pinfo in lookups.iteritems():
            owner = pinfo['comment']
            common = pinfo['body']['national_format']
            carrier = pinfo['body']['carrier']['name']
            pntype = pinfo['body']['carrier']['type']
            info('  {0}: {1}'.format(owner, common))
            info('    - carrier: {0}'.format(carrier))
            info('    - type: {0}'.format(pntype))
            debug('    - unformatted: {0}'.format(number))
            if 'sms' not in lookups[number]:
                carriers = ctx.obj['config']['sms_gateways'].keys()
                fuzzy = process.extract(carrier, carriers)
                fuzz_max = 0
                result = []
                gateway_key = None
                for f in fuzzy:
                    if f[1] >= fuzz_max:
                        fuzz_max = f[1]
                        result.append(f[0])
                debug(result)
                if len(result) > 1:
                    if len(sorted(result)[0]) <= len(carrier):
                        gateway_key = sorted(result)[0]
                    else:
                        gateway_key = result[0]
                else:
                    gateway_key = result[0]
                gw_suffix = ctx.obj['config']['sms_gateways'][gateway_key]
                ctx.obj['config']['lookups'][number]['sms'] = '{0}{1}'.format(
                    number, gw_suffix)
                sms_updated = True
            info('    - SMS gateway: {0}'.format(
                ctx.obj['config']['lookups'][number]['sms']))
        if sms_updated:
            with open(CONFIG_FILE, 'wb') as cfg:
                cfg.write(json.dumps(ctx.obj['config'], indent=2))
        ctx.exit()


helptxt['comment'] = 'add a comment to the phone number record (recommended)'
helptxt['nocache'] = 'do not cache the results into the config file'


@click.command()
@click.option(
    '-c', '--comment', 'comment',
    help=helptxt.get('comment'))
@click.option(
    '--no-cache', 'cache',
    is_flag=True, default=True,
    help=helptxt.get('nocache'))
@click.argument('number')
@click.pass_context
def lookup(ctx, number, comment, cache):
    """Get the carrier and country code for a phone number"""
    phone = PhoneNumber(number, comment=comment)
    info('{0} | {1}'.format(phone.number, ctx.obj['config']['lookups'].keys()))
    if phone.number in ctx.obj['config']['lookups']:
        info('{0} is already cached:'.format(phone.number))
        info(jsonify(ctx.obj['config']['lookups'][phone.number]))
        return
    data = phone.lookup()
    info('carrier = {0}'.format(phone.carrier))
    info('type = {0}'.format(phone.type))
    info('cache = {0}'.format(cache))
    if cache:
        ctx.obj['config']['lookups'][phone.number] = phone.raw
        with open(CONFIG_FILE, 'wb') as cfg:
            cfg.write(json.dumps(ctx.obj['config'], indent=2))

CLI.add_command(lookup)


helptxt['phone_number'] = '10 digit US/CA phone number'
helptxt['send_all'] = 'message is sent to all cached numbers (-n is ignored)'
helptxt['sms_message'] = 'SMS message to be sent to recipients (required)'


@click.command()
@click.option(
    '-n', '--number', 'numbers',
    multiple=True,
    help=helptxt.get('phone_number'))
@click.option(
    '--all', 'send_all',
    is_flag=True, default=False,
    help=helptxt.get('send_all'))
@click.option(
    '-m', '--message', 'message',
    required=True,
    help=helptxt.get('sms_message'))
@click.pass_context
def sms(ctx, numbers, send_all, message):
    smtp_login = ctx.obj['config'].get('smtp_login', None)
    if smtp_login is None:
        error('cannot send SMS messages until SMTP has been configured')
        return
    debug(numbers)
    debug('message="{0}"'.format(message))
    debug(jsonify(smtp_login))
    if len(message) > 160:
        info('converting to multipart message...')
        return
    lookups = ctx.obj['config'].get('lookups')
    if send_all:
        info('sending to all stored phone numbers...')
        with EmailSMS(**smtp_login) as smsgw:
            for num, data in lookups.iteritems():
                info('sending to {0}[{1}]'.format(
                    data['comment'], data['sms']))
                smsgw.send(data['sms'], message)
    elif len(numbers):
        info('sending to numbers: {0}'.format(numbers))
        with EmailSMS(**smtp_login) as smsgw:
            for num in numbers:
                data = lookups.get(num, None)
                if not data:
                    info('WARNING | unknown number: {0}'.format(num))
                    continue
                info('sending to {0}[{1}]'.format(
                    data['comment'], data['sms']))
                smsgw.send(data['sms'], message)
    else:
        info('no numbers have been selected!')

CLI.add_command(sms)


helptxt['config_server'] = 'SMTP server which should be used to proxy SMS'
helptxt['config_uname'] = 'SMTP server login username'
helptxt['config_pass'] = 'SMTP server login passcode'


@click.command()
@click.option(
    '-s', '--server', 'server',
    help=helptxt.get('config_server'))
@click.option(
    '-u', '--user', 'username',
    help=helptxt.get('config_uname'))
@click.option(
    '-p', '--pass', 'passcode',
    help=helptxt.get('config_pass'))
@click.pass_context
def config(ctx, server, username, passcode):
    smtp_login = ctx.obj['config'].get('smtp_login', None)
    if smtp_login is None:
        ctx.obj['config']['smtp_login'] = OrderedDict()
        smtp_login['server'] = None
        smtp_login['username'] = None
        smtp_login['passcode'] = None
        smtp_login = ctx.obj['config']['smtp_login']
    debug(jsonify(ctx.obj['config']['smtp_login']))

    update_config = False
    if server is not None and server != smtp_login['server']:
        smtp_login['server'] = server
        info('SMTP server set to: {0}'.format(server))
        update_config = True
    elif server is not None:
        info('SMTP server is already set to: {0}'.format(server))
    if username is not None and username != smtp_login['username']:
        smtp_login['username'] = username
        info('SMTP username set to: {0}'.format(username))
        update_config = True
    elif username is not None:
        info('SMTP username is already set to: {0}'.format(username))
    if passcode is not None and passcode != smtp_login['passcode']:
        smtp_login['passcode'] = passcode
        info('SMTP passcode set to: {0}'.format(passcode))
        update_config = True
    elif passcode is not None:
        info('SMTP passcode is already set to: {0}'.format(passcode))

    if not update_config:
        info('no config changes to write')
        return
    with open(CONFIG_FILE, 'wb') as config:
        config.write(jsonify(ctx.obj['config']))
    info('\nSMTP settings:')
    info('  - server = {0}'.format(ctx.obj['config']['smtp_login']['server']))
    info('  - user = {0}'.format(ctx.obj['config']['smtp_login']['username']))
    info('  - pass = {0}'.format(ctx.obj['config']['smtp_login']['passcode']))

CLI.add_command(config)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function

TRUTHS = ['yes', 'y', 'true', 't', '1']
FALSEHOODS = ['no', 'n', 'false', 'f', '0']


def boolean(value):
    """
    This function returns a boolean result based on the given value

    Unlike the 'bool' python builtin, this supports string values as well,
    which semantically indicate a value of truth or falehoods; these are
    'yes', 'y', 'true', 't', '1', 'no', 'n', 'false', 'f', '0'. If you do
    not need string support, then use the python builtin as usual.
    """
    try:
        val = str(value).lower()
        if val in TRUTHS:
            return True
        if val in FALSEHOODS:
            return False
    except (UnicodeDecodeError, TypeError):
        pass
    return bool(value)



# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function

"""
The Hub object in gevent processes exceptions raised by Greenlets, however,
under certain circumstances the system exception would be printed to console
even though it is simply supposed to be thrown upwards to let the external
code deal with it. This was due to how the control structure in the error
handling function was implemented; the function below patches the Hub object
to resolve this undesirable behavior.


def monkey_patch():
    patch_gevent_hub()
    gevent.monkey.patch_all()
    return True


class CrawlerGroup(Group):
    # gevent Group with extra method to determine greenlet status.
    def alive(self):
        for greenlet in self.greenlets:
            if not greenlet.ready():
                return True
        return False


def _manage_processor_pool(crawlers):
    while crawlers.alive() or not result_queue.empty():
        # keep the worker pool filled
        for x in range(0, min(result_queue.qsize(), processors.free_count())):
            processors.spawn(_scrape_processor)
        # ensure context switches can happen
        gevent.sleep(0.1)

"""


def patch_gevent_hub():
    """ This patches the error handler in the gevent Hub object. """
    from gevent.hub import Hub

    def patched_handle_error(self, context, etype, value, tb):
        """ Patched to not print KeyboardInterrupt exceptions. """
        if isinstance(value, str):
            value = etype(value)
        not_error = issubclass(etype, self.NOT_ERROR)
        system_error = issubclass(etype, self.SYSTEM_ERROR)
        if not not_error and not issubclass(etype, KeyboardInterrupt):
            self.print_exception(context, etype, value, tb)
        if context is None or system_error:
            self.handle_system_error(etype, value)

    Hub._original_handle_error = Hub.handle_error
    Hub.handle_error = patched_handle_error



from __future__ import absolute_import, unicode_literals
from __future__ import division, print_function

import uuid

from datetime import datetime
from django import forms
from crispy_forms import layout, helper, bootstrap
from crispy_forms.utils import TEMPLATE_PACK


class RangeWidget(forms.widgets.MultiWidget):
    """ django widget for displaying values representing a range """

    def __init__(self, widgets_from=None, widgets_to=None, **kwargs):
        """ `widgets_from` and `widgets_to` should be instantiated widgets """
        attrs_from = kwargs.pop('attrs_from', {})
        attrs_to = kwargs.pop('attrs_to', {})
        widget_class = kwargs.pop('widget_class', None)
        if widget_class is None and widgets_from is None:
            raise ValueError('either widget_class or widgets_from is required')
        if widget_class is None and widgets_to is None:
            raise ValueError('either widget_class or widgets_to is required')
        if widgets_from is None:
            widgets_from = [widget_class(attrs=attrs_from)]
        if widgets_to is None:
            widgets_to = [widget_class(attrs=attrs_to)]
        range_widgets = []
        try:
            iter(widgets_from)
        except TypeError:
            widgets_from = [widgets_from]
        try:
            iter(widgets_to)
        except TypeError:
            widgets_to = [widgets_to]
        range_widgets.extend(widgets_from)
        range_widgets.extend(widgets_to)
        self._from_to_divisor = len(widgets_from)
        super(RangeWidget, self).__init__(widgets=range_widgets, **kwargs)

    def decompress(self, value):
        if isinstance(value, (list, tuple)):
            return value
        if value is None:
            return []
        return (value, value)

    def format_output(self, rendered_widgets):
        lower = rendered_widgets[:self._from_to_divisor]
        upper = rendered_widgets[self._from_to_divisor:]
        return str().join([self.addon_from] + lower + [self.addon_to] + upper)

    def build_input_addon(self, content=None, elem=None, classes=None):
        css_class = 'input-group-addon'
        addon = '<{elem} class="{classes}">{content}</{elem}>'
        content = content if content is not None else ''
        elem = elem if elem is not None else 'span'
        if classes is None:
            addon_classes = css_class
        else:
            addon_classes = ' '.join([css_class, classes])
        return addon.format(elem=elem, classes=addon_classes, content=content)

    @property
    def addon_from(self):
        if not hasattr(self, '_addon_from'):
            self._addon_from = self.build_input_addon('from')
        return self._addon_from

    @property
    def addon_to(self):
        if not hasattr(self, '_addon_to'):
            self._addon_to = self.build_input_addon('to')
        return self._addon_to

    @property
    def addon_date(self):
        if not hasattr(self, '_addon_date'):
            self._addon_date = ''
        return self._addon_date

    @property
    def addon_time(self):
        if not hasattr(self, '_addon_time'):
            self._addon_time = ''
        return self._addon_time

    @property
    def addon_datetime(self):
        if not hasattr(self, '_addon_datetime'):
            self._addon_datetime = ''
        return self._addon_datetime


class RangeField(forms.fields.MultiValueField):
    """ django form field for displaying two values representing a range """

    def __init__(self, field_class, widget_class=None, *args, **kwargs):
        self.field_class = field_class
        self.widget_class = widget_class
        init_from = kwargs.pop('init_from', {})
        attrs_from = init_from.pop('attrs', {})
        init_to = kwargs.pop('init_to', {})
        attrs_to = init_to.pop('attrs', {})
        if widget_class:
            init_from['widget'] = widget_class(attrs=attrs_from)
            init_to['widget'] = widget_class(attrs=attrs_to)
        field_from = field_class(**init_from)
        field_to = field_class(**init_to)
        if not widget_class:
            field_from.widget.attrs.update(attrs_from)
            field_to.widget.attrs.update(attrs_to)
        if isinstance(field_from.widget, forms.MultiWidget):
            widgets_from = field_from.widget.widgets
        else:
            widgets_from = field_from.widget
        if isinstance(field_to.widget, forms.MultiWidget):
            widgets_to = field_to.widget.widgets
        else:
            widgets_to = field_to.widget
        range_widget = RangeWidget(widgets_from, widgets_to)
        kwargs['initial'] = (
            init_from.get('initial', None),
            init_to.get('initial', None),
        )
        super(RangeField, self).__init__(
            fields=(field_from, field_to),
            widget=range_widget,
            *args, **kwargs
        )

    def compress(self, data_list):
        if data_list:
            lower = self.fields[0].clean(data_list[0])
            upper = self.fields[1].clean(data_list[1])
            return (lower, upper)
        return tuple()


class IntegerRangeField(RangeField):
    def __init__(self, *args, **kwargs):
        super(IntegerRangeField, self).__init__(
            field_class=forms.fields.IntegerField,
            widget_class=forms.widgets.NumberInput,
            *args, **kwargs
        )


class DateTimeRangeWidget(RangeWidget):
    """ django widget for displaying datetime values representing a range """

    def format_output(self, rendered_widgets):
        lower = rendered_widgets[:self._from_to_divisor]
        upper = rendered_widgets[self._from_to_divisor:]
        if len(lower) > 1:
            lower.insert(1, self.addon_date)
            lower.append(self.addon_time)
        else:
            lower.append(self.addon_datetime)
        if len(upper) > 1:
            upper.insert(1, self.addon_date)
            upper.append(self.addon_time)
        else:
            upper.append(self.addon_datetime)
        return str().join([self.addon_from] + lower + [self.addon_to] + upper)

    @property
    def addon_date(self):
        if not hasattr(self, '_addon_date'):
            date_classes = 'glyphicon glyphicon-date'
            self._addon_date = self.build_input_addon(classes=date_classes)
        return self._addon_date

    @property
    def addon_time(self):
        if not hasattr(self, '_addon_time'):
            time_classes = 'glyphicon glyphicon-time'
            self._addon_time = self.build_input_addon(classes=time_classes)
        return self._addon_time

    @property
    def addon_datetime(self):
        if not hasattr(self, '_addon_datetime'):
            dt_classes = 'glyphicon glyphicon-datetime'
            self._addon_datetime = self.build_input_addon(classes=dt_classes)
        return self._addon_datetime


class DateTimeRangeField(RangeField):
    widget = DateTimeRangeWidget

    def __init__(self, split=False, *args, **kwargs):
        self._split = split
        if split:
            field_class = forms.SplitDateTimeField
            widget_class = forms.SplitDateTimeWidget
        else:
            field_class = forms.DateTimeField
            widget_class = forms.DateTimeInput
        super(DateTimeRangeField, self).__init__(
            field_class=field_class,
            widget_class=widget_class,
            *args, **kwargs
        )

    def clean(self, value):
        value = super(DateTimeRangeField, self).clean(value)
        return value

    def compress(self, data_list):
        if data_list:
            raise ValueError(data_list)
            return [datetime.Date(v) for v in data_list]
        return None


class CrispyField(layout.Field):
    """ crispy_forms Field overrides """
    def __init__(self, *args, **kwargs):
        if 'label_class' in kwargs:
            self.label_class = kwargs.pop('label_class')
        if 'field_class' in kwargs:
            self.field_class = kwargs.pop('field_class')
        super(CrispyField, self).__init__(*args, **kwargs)

    def render(
            self, form, form_style, context,
            template_pack=TEMPLATE_PACK, extra_context=None, **kwargs):
        if extra_context is None:
            extra_context = {}
        if hasattr(self, 'label_class'):
            extra_context['label_class'] = self.label_class
        if hasattr(self, 'field_class'):
            extra_context['field_class'] = self.field_class
        return super(CrispyField, self).render(
            form, form_style, context, template_pack, extra_context, **kwargs
        )


class CrispyMultiWidgetField(CrispyField):
    def __init__(self, *args, **kwargs):
        if 'wrapper_class' in kwargs:
            self.wrapper_class = kwargs.pop('wrapper_class')
        if 'label_class' in kwargs:
            self.label_class = kwargs.pop('label_class')
        if 'field_class' in kwargs:
            self.field_class = kwargs.pop('field_class')
        self.fields = list(args)
        self.attrs = kwargs.pop('attrs', {})
        self.template = kwargs.pop('template', self.template)


class PanelFormHelper(helper.FormHelper):
    """helper specifically for creating bootstrap 3 forms within Panels"""
    PANELTYPES = ('default', 'primary', 'success', 'info', 'warning', 'danger')

    def __init__(self, title, panel=None, **options):
        super(PanelFormHelper, self).__init__()
        self._layout = None
        self._form_collapse = options.get('collapse', False)
        self.form_method = options.get('method', 'POST')
        if panel is None:
            panel = 'default'
        if panel not in self.PANELTYPES:
            errmsg = '{0} is not a valid bootstrap panel: valid panels are {1}'
            raise ValueError(errmsg.format(panel, self.PANELTYPES))
        self._panel_class = '{0} panel-{1}'.format('panel', panel)
        self._panel_title = None
        if title is not None:
            h1_title = '<h1 class="panel-title">{0}</h1>'
            self._panel_title = h1_title.format(title)
        self.form_class = 'form form-horizontal'
        self.label_class = 'col-lg-2 col-md-2 col-xs-2'
        self.field_class = 'col-lg-8 col-md-9 col-xs-10'
        form = options.get('form', None)
        self._form = form
        form_layout = options.get('layout', None)
        if form_layout is not None:
            self.layout = form_layout
        elif form is not None:
            form_actions = options.get('form_actions', [])
            self.layout = self.default_bootstrap_layout(form, form_actions)

    def default_bootstrap_layout(self, form, actions=None):
        actions = [] if actions is None else actions
        form_fields = []
        small_fields = (
            forms.ChoiceField, forms.IntegerField, forms.DateTimeField
        )
        for name, field in form.fields.items():
            attrs = {}
            if isinstance(field, small_fields):
                attrs['field_class'] = 'col-lg-2 col-md-3 col-xs-4'
            if isinstance(field.widget, forms.widgets.MultiWidget):
                form_fields.append(
                    CrispyMultiWidgetField(name, wrapper_class='multifield')
                )
            else:
                form_fields.append(CrispyField(name, **attrs))
        form_layout = layout.Layout(*form_fields)
        form_actions = []
        for action in actions:
            if action['type'] == 'submit':
                args = {}
                if 'class' in action:
                    args['css_class'] = 'btn-{0}'.format(action['class'])
                form_actions.append(layout.Submit(
                    action['name'],
                    action['value'],
                    **args
                ))
            if action['type'] == 'reset':
                args = {}
                if 'class' in action:
                    args['css_class'] = 'btn-{0}'.format(action['class'])
                form_actions.append(layout.Reset(
                    action['name'],
                    action['value'],
                    **args
                ))
        if not actions:
            form_actions.append(layout.Submit('submit', 'Submit'))
        form_layout.append(bootstrap.FormActions(*form_actions))
        return form_layout

    @property
    def layout(self):
        return self._layout

    @layout.setter
    def layout(self, value):
        self._layout = value
        panel_id = getattr(self, 'form_panel_id', None)
        panel_id = uuid.uuid4().hex if panel_id is None else panel_id
        collapse = ' in' if self._form_collapse else ''
        self.all().wrap_together(
            layout.Div,
            css_class='panel-body{0}'.format(collapse),
            id='{0}'.format(panel_id)
        )
        collapse_options = {}
        if self._form_collapse:
            collapse_options['data_toggle'] = 'collapse'
            collapse_options['data_target'] = '#{0}'.format(panel_id)
            collapse_options['aria_expanded'] = 'true'
        if self._panel_title is not None:
            self._layout.insert(0, layout.Div(
                layout.HTML(self._panel_title),
                css_class='panel-heading',
                **collapse_options
            ))
        self.all().wrap_together(layout.Div, css_class=self._panel_class)


class BootstrapPanelForm(forms.Form):
    """
    Base for bootstrap panel forms which use django-crispy-forms

    Attributes you can set on inherited class:
        ``form_title``: sets the bootstrap panel heading (supports jinja2)
        ``form_context``: sets the bootstrap context coloring for the panel
        ``form_method``: sets the form method (defaults to 'post')
        ``form_id``: sets the DOM id for the form
        ``form_collapse``: whether the form is collapsible (default: False)
        ``form_panel_id``: sets the DOM id for the panel body of the form
    """

    @property
    def helper(self):
        if not hasattr(self, '_helper'):
            self._helper = self.form_setup()
        return self._helper

    def form_setup(self):
        pfh_options = {
            'title': getattr(self, 'form_title', ''),
            'panel': getattr(self, 'form_context', 'default'),
            'method': getattr(self, 'form_method', 'post'),
            'collapse': getattr(self, 'form_collapse', False),
            'form': self,
        }
        if hasattr(self, 'form_actions'):
            pfh_options['form_actions'] = self.form_actions
        pfh = PanelFormHelper(**pfh_options)
        if hasattr(self, 'form_id'):
            pfh.form_id = self.form_id
        if hasattr(self, 'form_panel_id'):
            pfh.form_panel_id = self.form_panel_id
        return pfh



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import io
import os
import re

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))


PROJECT_MODULE = 'armory'
PROJECT = 'armory'
AUTHOR = 'Bryce Eggleton'
EMAIL = 'eggleton.bryce@gmail.com'
DESC = 'Python utilities and tools'
LONG_DESC = ''
URL = "https://github.com/neuroticnerd/armory"
REQUIRES = []
SCRIPTS = {
    "console_scripts": [
        'phone = armory.phone.cli:CLI',
    ]}
LICENSE = 'BSD'
VERSION = ''

version_file = os.path.join(here, '{0}/__version__.py'.format(PROJECT_MODULE))
ver_find = r'^\s*__version__\s*=\s*[\"\'](.*)[\"\']$'
with io.open(version_file, 'r', encoding='utf-8') as ver_file:
    VERSION = re.search(ver_find, ver_file.read(), re.MULTILINE).group(1)

readme_file = os.path.join(here, 'README.rst')
with io.open(readme_file, 'r', encoding='utf-8') as f:
    LONG_DESC = f.read()

if __name__ == '__main__':
    setup(
        name=PROJECT,
        version=VERSION,
        packages=find_packages(include=[PROJECT_MODULE + '*']),
        author=AUTHOR,
        author_email=EMAIL,
        url=URL,
        description=DESC,
        long_description=LONG_DESC,
        license=LICENSE,
        install_requires=REQUIRES,
        # entry_points = SCRIPTS,
    )



# -*- encoding: utf-8 -*-
from __future__ import absolute_import

import re


class UnicodeTransformChar(object):
    def __init__(self, char_regex, replacement):
        """Initializes with the regex and replacement substitution

        !important: regex encoding must match the replacement and ``text``
        parameter encoding when calling the :func:`transform` method!
        """
        self.regex = re.compile(char_regex)
        self.repl = replacement

    def transform(self, text):
        """Replaces characters in string ``text`` based in regex sub"""
        return re.sub(self.regex, self.repl, text)



# -*- encoding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from armory.phone.cli import CLI
from armory.phone.phone import PhoneNumber
from armory.phone.lookup import carrier_lookup, CarrierLookup

DEFAULT_GATEWAYS = {
    "sms_gateways": {
        "3 River Wireless": "@sms.3rivers.net",
        "ACS Wireless": "@paging.acswireless.com",
        "AT&T": "@txt.att.net",
        "Alltel": "@message.alltel.com",
        "BPL Mobile": "@bplmobile.com",
        "Bell Canada": "@bellmobility.ca",
        "Bell Mobility": "@txt.bellmobility.ca",
        "Bell Mobility (Canada)": "@txt.bell.ca",
        "Blue Sky Frog": "@blueskyfrog.com",
        "Bluegrass Cellular": "@sms.bluecell.com",
        "Boost Mobile": "@myboostmobile.com",
        "Carolina West Wireless": "@cwwsms.com",
        "Cellular One": "@mobile.celloneusa.com",
        "Cellular South": "@csouth1.com",
        "Centennial Wireless": "@cwemail.com",
        "CenturyTel": "@messaging.centurytel.net",
        "Cingular (Now AT&T)": "@txt.att.net",
        "Clearnet": "@msg.clearnet.com",
        "Comcast": "@comcastpcs.textmsg.com",
        "Corr Wireless Communications": "@corrwireless.net",
        "Dobson": "@mobile.dobson.net",
        "Edge Wireless": "@sms.edgewireless.com",
        "Fido": "@fido.ca",
        "Golden Telecom": "@sms.goldentele.com",
        "Helio": "@messaging.sprintpcs.com",
        "Houston Cellular": "@text.houstoncellular.net",
        "Idea Cellular": "@ideacellular.net",
        "Illinois Valley Cellular": "@ivctext.com",
        "Inland Cellular Telephone": "@inlandlink.com",
        "MCI": "@pagemci.com",
        "MTS": "@text.mtsmobility.com",
        "Metro PCS": "@mymetropcs.com",
        "Microcell": "@fido.ca",
        "Midwest Wireless": "@clearlydigital.com",
        "Mobilcomm": "@mobilecomm.net",
        "Nextel": "@messaging.nextel.com",
        "OnlineBeep": "@onlinebeep.net",
        "PCS One": "@pcsone.net",
        "President's Choice": "@txt.bell.ca",
        "Public Service Cellular": "@sms.pscel.com",
        "Qwest": "@qwestmp.com",
        "Rogers AT&T Wireless": "@pcs.rogers.com",
        "Rogers Canada": "@pcs.rogers.com",
        "Solo Mobile": "@txt.bell.ca",
        "Southwestern Bell": "@email.swbw.com",
        "Sprint": "@messaging.sprintpcs.com",
        "Sumcom": "@tms.suncom.com",
        "Surewest Communicaitons": "@mobile.surewest.com",
        "T-Mobile": "@tmomail.net",
        "Telus": "@msg.telus.com",
        "Tracfone": "@txt.att.net",
        "Triton": "@tms.suncom.com",
        "US Cellular": "@email.uscc.net",
        "US West": "@uswestdatamail.com",
        "Unicel": "@utext.com",
        "Verizon": "@vtext.com",
        "Virgin Mobile": "@vmobl.com",
        "Virgin Mobile Canada": "@vmobile.ca",
        "West Central Wireless": "@sms.wcc.net",
        "Western Wireless": "@cellularonewest.com",
    },
}



# -*- encoding: utf-8 -*-
from __future__ import absolute_import

import logging
import smtplib

from armory.phone.lookup import carrier_lookup
from armory.serialize import jsonify


class PhoneNumber(object):
    """lookup country code and carrier for a phone number"""
    def __init__(self, phonenumber=None, data=None, logger=None, comment=None):
        self._logger = logger if logger else ''
        self.log = logging.getLogger(self._logger)
        self._raw = None
        self._carrier = None
        self._type = None
        self._comment = None
        self.number = ''
        if data is not None:
            self._raw = data
            self._unpack()
        if phonenumber is not None:
            num = phonenumber.strip()
            num = num.replace('(', '')
            num = num.replace(')', '')
            num = num.replace('-', '')
            num = num.replace(' ', '')
            if len(num) != 10:
                raise ValueError('phone number is not 10 digits')
            self.number = num
        if comment is not None:
            self._comment = comment

    @property
    def raw(self):
        if not self._raw:
            self.lookup()
        return self._raw

    @property
    def carrier(self):
        if not self._carrier:
            self.lookup()
        return self._carrier

    @property
    def type(self):
        if not self._type:
            self.lookup()
        return self._type

    def _unpack(self):
        if not self._raw:
            return
        raw = self._raw
        success = raw.get('success')
        if not success:
            raise ValueError('raw data could not be unpacked')
        data = raw.get('body')
        carrier = data.get('carrier')
        self._carrier = carrier.get('name')
        self._type = carrier.get('type')

    def lookup(self):
        log = self.log
        try:
            info = carrier_lookup(self.number, self._logger)
        except ValueError:
            raise SystemExit()
        log.debug(jsonify(info))
        self._raw = info
        self._unpack()
        self._raw['comment'] = self._comment
        return info


# http://stackoverflow.com/questions/9763455/how-to-send-a-mail-directly-to-smtp-server-without-authentication
class EmailSMS(object):
    def __init__(self, server, username, passcode, sender=None, logger=None):
        logger = logger if logger else __name__
        self.log = logging.getLogger(logger)
        self._smtp = None
        self._smtp_tls = True
        self._smtp_login = True
        self._server = server
        self._smtp_user = username
        self._passcode = passcode
        self._sender = sender if sender else self._smtp_user

    def __enter__(self):
        self._initialize_smtp()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self._smtp:
            self._smtp.quit()
        self._smtp = None

    def _initialize_smtp(self):
        self._smtp = smtplib.SMTP(self._server)
        self._smtp.starttls()
        self._smtp.login(self._smtp_user, self._passcode)

    @property
    def smtp(self):
        if not self._smtp:
            self._initialize_smtp()
        return self._smtp

    def send(self, recipient, message):
        self.smtp.sendmail(self._sender, recipient, message)
        self.log.info('sender: {0}'.format(self._sender))
        self.log.info('recipient: {0}'.format(recipient))
        self.log.info('message: "{0}"'.format(message))
        self.log.info('')
